﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace net.wafly.tools {
    /// <summary>
    /// 压缩包处理器
    /// 作者：<a href="http://wafly.net">wafly.net</a>
    /// </summary>
    public class Compressor {
        /// <summary>
        /// 支持的压缩格式
        /// </summary>
        public enum CompressType { ZIP, SEVEN_ZIP, RAR, CAB, G_ZIP, B_ZIP_2, TAR }
        /// <summary>
        /// 进度状态处理机，进度值（0.0 ～ 1.0），用于实时更新处理进度
        /// </summary>
        public interface ProgressActor { void OnProgressChanged(double progressValue); }
        /// <summary>
        /// 异常信息处理机，用于显示正在处理中的错误信息
        /// </summary>
        public interface ErrorActor { void OnError(string message); }

        private const string excutorName = "compress.dll";
        private static Dictionary<CompressType, string> compressCodes = new Dictionary<CompressType, string>() {
             {CompressType.ZIP, "zip"}, {CompressType.SEVEN_ZIP, "7z"}, {CompressType.RAR, "rar"}, {CompressType.CAB, "cab"}, {CompressType.G_ZIP, "gzip"}, {CompressType.B_ZIP_2, "bzip2"}, {CompressType.TAR, "tar"}
        };

        private string compressType;
        private string zipExcutorPath;
        /// <summary>
        /// 初始化一个特定类型的压缩包处理器
        /// </summary>
        /// <param name="compressType"></param>
        public Compressor(CompressType compressType) {
            zipExcutorPath = Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName) + "\\" + excutorName;
            if (!File.Exists(zipExcutorPath)) {
                throw new Exception("执行目录下没有找到" + excutorName + "文件");
            }
            this.compressType = compressCodes[compressType];
        }
        /// <summary>
        /// 压缩
        /// </summary>
        /// <param name="fileMap">key=压缩包内显示的文件名(含目录)，value=要压缩的文件全路径</param>
        /// <param name="target">要压缩到的目标压缩包文件，已存在则往其中追加文件</param>
        /// <param name="progressActor">进度状态处理机，进度值（0.0 ～ 1.0），可以为null</param>
        /// <param name="errorActor">异常信息处理机，可以为null</param>
        public void Compress(Dictionary<string, string> fileMap, string target, ProgressActor progressActor, ErrorActor errorActor) {
            if (fileMap == null || fileMap.Count == 0) {
                errorActor.OnError("要压缩的源文件列表为空");
                return;
            }
            if (target == null || target.Trim().Length == 0) {
                errorActor.OnError("要压缩到的目标压缩包文件为空");
                return;
            }
            string oriPath = Directory.GetCurrentDirectory();
            string tmpFolder = Path.GetTempPath() + "\\" + DateTime.Now.Ticks + "-" + new Random().Next(1, 10000000);
            Directory.CreateDirectory(tmpFolder);
            Directory.SetCurrentDirectory(tmpFolder);
            int handledFiles = 0;
            foreach (KeyValuePair<string, string> files in fileMap) {
                try {
                    string targetFolder = Path.GetDirectoryName(files.Key);
                    if (targetFolder != null && targetFolder.Trim().Length > 0) {
                        Directory.CreateDirectory(targetFolder);
                    }
                    File.Copy(files.Value, files.Key);
                    Process proc = new Process();
                    proc.StartInfo.FileName = zipExcutorPath;
                    proc.StartInfo.Arguments = " a -t" + compressType + " \"" + target + "\" \"" + files.Key + "\"";
                    proc.StartInfo.UseShellExecute = false;
                    proc.StartInfo.RedirectStandardError = true;
                    proc.StartInfo.RedirectStandardOutput = true;
                    proc.StartInfo.CreateNoWindow = true;
                    proc.Start();
                    proc.WaitForExit();
                    proc.Dispose();
                    File.Delete(files.Key);
                } catch (Exception ex) {
                    if (errorActor != null) {
                        errorActor.OnError(files + ": " + ex.Message);
                    }
                }
                if (progressActor != null) {
                    progressActor.OnProgressChanged(++handledFiles / (double)fileMap.Count);
                }
            }
            Directory.SetCurrentDirectory(oriPath);
            Directory.Delete(tmpFolder, true);
        }
        /// <summary>
        /// 压缩
        /// </summary>
        /// <param name="fileMap">key=压缩包内显示的文件名(含目录)，value=要压缩的文件全路径</param>
        /// <param name="target">要压缩到的目标压缩包文件，已存在则往其中追加文件</param>
        /// <param name="progressActor">进度状态处理机，进度值（0.0 ～ 1.0），可以为null</param>
        /// <param name="errorActor">异常信息处理机，可以为null</param>
        public void Compress(Dictionary<string, string> fileMap, string target, ErrorActor errorActor) {
            if (fileMap == null || fileMap.Count == 0) {
                errorActor.OnError("要压缩的源文件列表为空");
                return;
            }
            if (target == null || target.Trim().Length == 0) {
                errorActor.OnError("要压缩到的目标压缩包文件为空");
                return;
            }
            string oriPath = Directory.GetCurrentDirectory();
            string tmpFolder = Path.GetTempPath() + "\\" + DateTime.Now.Ticks + "-" + new Random().Next(1, 10000000);
            Directory.CreateDirectory(tmpFolder);
            Directory.SetCurrentDirectory(tmpFolder);
            foreach (KeyValuePair<string, string> files in fileMap) {
                try {
                    string targetFolder = Path.GetDirectoryName(files.Key);
                    if (targetFolder != null && targetFolder.Trim().Length > 0) {
                        Directory.CreateDirectory(targetFolder);
                    }
                    File.Copy(files.Value, files.Key);
                } catch (Exception ex) {
                    if (errorActor != null) {
                        errorActor.OnError(files + ": " + ex.Message);
                    }
                }
            }
            StringBuilder filesSb = new StringBuilder();
            foreach (string f in fileMap.Keys) {
                filesSb.Append(" \"").Append(f).Append("\"");
            }
            try {
                Process proc = new Process();
                proc.StartInfo.FileName = zipExcutorPath;
                proc.StartInfo.Arguments = " a -t" + compressType + " \"" + target + "\" " + filesSb;
                proc.StartInfo.UseShellExecute = false;
                proc.StartInfo.RedirectStandardError = true;
                proc.StartInfo.RedirectStandardOutput = true;
                proc.StartInfo.CreateNoWindow = true;
                proc.Start();
                proc.WaitForExit();
                proc.Dispose();
                Directory.SetCurrentDirectory(oriPath);
                Directory.Delete(tmpFolder, true);
            } catch (Exception ex) {
                if (errorActor != null) {
                    errorActor.OnError(ex.Message);
                }
            }
        }
        /// <summary>
        /// 解压
        /// </summary>
        /// <param name="compressFilePath">要解压的压缩包文件</param>
        /// <param name="destDirectory">要解压到的目录</param>
        /// <param name="errorActor">异常信息处理机，可以为null</param>
        public void DeCompress(string compressFilePath, string destDirectory, ErrorActor errorActor) {
            try {
                Directory.CreateDirectory(destDirectory);
                Process proc = new Process();
                proc.StartInfo.FileName = zipExcutorPath;
                proc.StartInfo.Arguments = " x \"" + compressFilePath + "\" -o\"" + destDirectory + "\"";
                proc.StartInfo.UseShellExecute = false;
                proc.StartInfo.RedirectStandardError = true;
                proc.StartInfo.RedirectStandardOutput = true;
                proc.StartInfo.CreateNoWindow = true;
                proc.Start();
                proc.WaitForExit();
                proc.Dispose();
            } catch (Exception ex) {
                if (errorActor != null) {
                    errorActor.OnError(ex.Message);
                }
            }
        }

    }
}
 